package gregapi.block.tree;

import gregapi.block.BlockBase;
import gregapi.render.IIconContainer;
import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.item.ItemBlock;
import net.minecraft.util.IIcon;
import net.minecraft.world.World;

/**
 * @author Gregorius Techneticies
 */
public abstract class BlockBaseTree extends BlockBase {
	public final IIconContainer[] mIcons;
	
	public BlockBaseTree(Class<? extends ItemBlock> aItemClass, String aNameInternal, Material aMaterial, SoundType aSoundType, IIconContainer[] aIcons) {
		super(aItemClass, aNameInternal, aMaterial, aSoundType);
		mIcons = aIcons;
	}
	
	@Override
	public IIcon getIcon(int aSide, int aMeta) {
		return mIcons[aMeta % mIcons.length].getIcon(0);
	}
	
	public abstract int getLeavesRange(byte aMeta);
	
	@Override
	public void breakBlock(World aWorld, int aX, int aY, int aZ, Block aBlock, int aMeta) {
		int tRange = getLeavesRange((byte)aMeta)+1;
		if (!aWorld.isRemote && aWorld.checkChunksExist(aX - tRange, aY - tRange, aZ - tRange, aX + tRange, aY + tRange, aZ + tRange)) {
			tRange--;
			for (int i = -tRange; i <= tRange; ++i) for (int j = -tRange; j <= tRange; ++j) for (int k = -tRange; k <= tRange; ++k) {
				Block tBlock = aWorld.getBlock(aX + i, aY + j, aZ + k);
				if (tBlock.isLeaves(aWorld, aX + i, aY + j, aZ + k)) tBlock.beginLeavesDecay(aWorld, aX + i, aY + j, aZ + k);
			}
		}
	}
}